searchData={"items":[{"type":"behaviour","title":"Fresh","doc":"This module provides a high-level interface for managing WebSocket connections.\n\nIt simplifies implementing WebSocket clients, allowing you to easily establish and manage connections with WebSocket servers.","ref":"Fresh.html"},{"type":"behaviour","title":"Usage - Fresh","doc":"To use this module, follow these steps:\n\n1. Use the `use Fresh` macro in your WebSocket client module to automatically configure the necessary callbacks and functionality:\n\n       defmodule MyWebSocketClient do\n         use Fresh\n\n         # ...callbacks and functionalities.\n       end\n\n2. Implement callback functions to handle connection events, received data frames, and more, as specified in the documentation.\n\n3. Start WebSocket connections using `start_link/1` or `start/1` with the desired configuration options:\n\n       MyWebSocketClient.start_link(uri: \"wss://example.com/socket\", state: %{}, opts: [\n         name: {:local, :my_connection}\n       ])","ref":"Fresh.html#module-usage"},{"type":"behaviour","title":"How to Supervise - Fresh","doc":"For effective management of WebSocket connections, consider supervising your WebSocket client processes.\nYou can add your WebSocket client module as a child to a supervisor, allowing the supervisor to monitor and restart the WebSocket client process in case of failures.\n\n    children = [\n      {MyWebSocketClient,\n       uri: \"wss://example.com/socket\",\n       state: %{},\n       opts: [\n         name: {:local, :my_connection}\n       ]}\n\n      # ...other child specifications\n    ]\n\n    Supervisor.start_link(children, strategy: :one_for_one)","ref":"Fresh.html#module-how-to-supervise"},{"type":"behaviour","title":"Reconnection - Fresh","doc":"In scenarios where the WebSocket connection is lost or encounters an error, you can configure reconnection behaviour using `c:handle_disconnect/3` and `c:handle_error/2` callbacks.\nDepending on your requirements, you can implement logic to automatically reconnect to the server or take other appropriate actions.","ref":"Fresh.html#module-reconnection"},{"type":"behaviour","title":"Automatic Reconnection and Backoff - Fresh","doc":"Fresh uses exponential backoff (with a fixed factor of `1.5`) strategy for reconnection attempts.\nThis means that after a connection loss, it waits for a brief interval before attempting to reconnect, gradually increasing the time between reconnection attempts until a maximum limit is reached.\n\nThe exponential backoff strategy helps prevent overwhelming the server with rapid reconnection attempts and allows for a more graceful recovery when the server is temporarily unavailable.\n\nThe default backoff parameters are as follows:\n\n* Initial Backoff Interval: 250 milliseconds\n* Maximum Backoff Interval: 30 seconds\n\nYou can customize these parameters by including them in your WebSocket client's configuration, as shown in the \"Example Configuration\" section of the `t:option/0` documentation.","ref":"Fresh.html#module-automatic-reconnection-and-backoff"},{"type":"macro","title":"Fresh.__using__/1","doc":"This macro simplifies the implementation of WebSocket client.\n\nIt automatically configures `child_spec/1`, `start/1` and `start_link/1` for the module, and provides handlers for all required callbacks, which can be overridden.","ref":"Fresh.html#__using__/1"},{"type":"function","title":"Fresh.close/3","doc":"Sends a WebSocket close frame to the server.","ref":"Fresh.html#close/3"},{"type":"function","title":"Parameters - Fresh.close/3","doc":"* `pid` - The reference to the WebSocket connection process.\n* `code` - An integer representing the WebSocket close code.\n* `reason` - A binary string providing the reason for closing the WebSocket connection.","ref":"Fresh.html#close/3-parameters"},{"type":"function","title":"Returns - Fresh.close/3","doc":"This function always returns `:ok`.","ref":"Fresh.html#close/3-returns"},{"type":"function","title":"Example - Fresh.close/3","doc":"iex> Fresh.close(:ws_conn, 1000, \"Normal Closure\")\n    :ok","ref":"Fresh.html#close/3-example"},{"type":"callback","title":"Fresh.handle_connect/3","doc":"Callback invoked when a WebSocket connection is successfully established.","ref":"Fresh.html#c:handle_connect/3"},{"type":"callback","title":"Parameters - Fresh.handle_connect/3","doc":"* `status` - The status received during the connection upgrade.\n* `headers` - The headers received during the connection upgrade.\n* `state` - The current state of the module.","ref":"Fresh.html#c:handle_connect/3-parameters"},{"type":"callback","title":"Example - Fresh.handle_connect/3","doc":"def handle_connect(_status, _headers, state) do\n      payload = \"connection up!\"\n      {:reply, [{:text, payload}], state}\n    end","ref":"Fresh.html#c:handle_connect/3-example"},{"type":"callback","title":"Fresh.handle_control/2","doc":"Callback invoked when a control frame is received from the server.","ref":"Fresh.html#c:handle_control/2"},{"type":"callback","title":"Parameters - Fresh.handle_control/2","doc":"* `frame` - The received WebSocket frame, which is a control frame.\n* `state` - The current state of the module.","ref":"Fresh.html#c:handle_control/2-parameters"},{"type":"callback","title":"Example - Fresh.handle_control/2","doc":"def handle_control({:ping, message}, state) do\n      IO.puts(\"Received ping with content: #{message}!\")\n      {:ok, state}\n    end\n\n    def handle_control({:pong, message}, state) do\n      IO.puts(\"Received pong with content: #{message}!\")\n      {:ok, state}\n    end","ref":"Fresh.html#c:handle_control/2-example"},{"type":"callback","title":"Fresh.handle_disconnect/3","doc":"Callback invoked when the WebSocket connection is being disconnected.","ref":"Fresh.html#c:handle_disconnect/3"},{"type":"callback","title":"Parameters - Fresh.handle_disconnect/3","doc":"* `code` (optional) - The disconnection code, if available. It should be a non-negative integer.\n* `reason` (optional) - The reason for the disconnection, if available. It should be a binary.\n* `state` - The current state of the module.","ref":"Fresh.html#c:handle_disconnect/3-parameters"},{"type":"callback","title":"Example - Fresh.handle_disconnect/3","doc":"def handle_disconnect(1002, _reason, _state), do: :reconnect\n    def handle_disconnect(_code, _reason, _state), do: :close","ref":"Fresh.html#c:handle_disconnect/3-example"},{"type":"callback","title":"Fresh.handle_error/2","doc":"Callback invoked when an error is encountered during WebSocket communication, allowing you to define custom error handling logic for various scenarios.","ref":"Fresh.html#c:handle_error/2"},{"type":"callback","title":"Parameters - Fresh.handle_error/2","doc":"* `error` - The encountered error.\n* `state` - The current state of the module.","ref":"Fresh.html#c:handle_error/2-parameters"},{"type":"callback","title":"Example - Fresh.handle_error/2","doc":"def handle_error({error, _reason}, state)\n        when error in [:encoding_failed, :casting_failed],\n        do: {:ignore, state}\n\n    def handle_error(_error, _state), do: :reconnect","ref":"Fresh.html#c:handle_error/2-example"},{"type":"callback","title":"Fresh.handle_in/2","doc":"Callback invoked when a data frame is received from the server.","ref":"Fresh.html#c:handle_in/2"},{"type":"callback","title":"Parameters - Fresh.handle_in/2","doc":"* `frame` - The received WebSocket frame, which is a data frame.\n* `state` - The current state of the module.","ref":"Fresh.html#c:handle_in/2-parameters"},{"type":"callback","title":"Example - Fresh.handle_in/2","doc":"def handle_in({:text, message}, state) do\n      %{\"data\" => updated_data} = Jason.decode!(message)\n      {:ok, updated_data}\n    end\n\n    def handle_in({:binary, _message}, state) do\n      {:reply, [{:text, \"i prefer text :)\"}], state}\n    end","ref":"Fresh.html#c:handle_in/2-example"},{"type":"callback","title":"Fresh.handle_info/2","doc":"Callback invoked when an incomprehensible message is received.","ref":"Fresh.html#c:handle_info/2"},{"type":"callback","title":"Parameters - Fresh.handle_info/2","doc":"* `data` - The received message, which can be any term.\n* `state` - The current state of the module.","ref":"Fresh.html#c:handle_info/2-parameters"},{"type":"callback","title":"Example - Fresh.handle_info/2","doc":"def handle_info({:reply, message}, state) do\n      {:reply, [{:text, message}], state}\n    end\n\n  Later can be used like:\n\n    send(:ws_conn, {:reply, \"hello!\"})","ref":"Fresh.html#c:handle_info/2-example"},{"type":"callback","title":"Fresh.handle_terminate/2","doc":"Callback invoked when the WebSocket process is about to terminate.\n\nThe return value of this callback is always disregarded.","ref":"Fresh.html#c:handle_terminate/2"},{"type":"callback","title":"Parameters - Fresh.handle_terminate/2","doc":"* `reason` - The reason for the termination. It can be any term.\n* `state` - The current state of the module.","ref":"Fresh.html#c:handle_terminate/2-parameters"},{"type":"callback","title":"Example - Fresh.handle_terminate/2","doc":"def handle_terminate(reason, _state) do\n      IO.puts(\"Process is terminating with reason: #{inspect(reason)}\")\n    end","ref":"Fresh.html#c:handle_terminate/2-example"},{"type":"function","title":"Fresh.open?/1","doc":"Checks if the connection is available.","ref":"Fresh.html#open?/1"},{"type":"function","title":"Parameters - Fresh.open?/1","doc":"* `pid` - The reference to the WebSocket connection process.","ref":"Fresh.html#open?/1-parameters"},{"type":"function","title":"Returns - Fresh.open?/1","doc":"A `t:boolean/0` representing the state of the connection.","ref":"Fresh.html#open?/1-returns"},{"type":"function","title":"Example - Fresh.open?/1","doc":"iex> Fresh.open?(:ws_conn)\n    true","ref":"Fresh.html#open?/1-example"},{"type":"function","title":"Fresh.send/2","doc":"Sends a WebSocket frame to the server.","ref":"Fresh.html#send/2"},{"type":"function","title":"Parameters - Fresh.send/2","doc":"* `pid` - The reference to the WebSocket connection process.\n* `frame` - The WebSocket frame to send.","ref":"Fresh.html#send/2-parameters"},{"type":"function","title":"Returns - Fresh.send/2","doc":"This function always returns `:ok`.","ref":"Fresh.html#send/2-returns"},{"type":"function","title":"Example - Fresh.send/2","doc":"iex> Fresh.send(:ws_conn, {:text, \"hi!\"})\n    :ok","ref":"Fresh.html#send/2-example"},{"type":"function","title":"Fresh.start/4","doc":"Starts a WebSocket connection without linking the process.\n\nThis function is similar to `start_link/4` but does not link the process. Refer to `start_link/4` for parameters details.","ref":"Fresh.html#start/4"},{"type":"function","title":"Fresh.start_link/4","doc":"Starts a WebSocket connection and links the process.","ref":"Fresh.html#start_link/4"},{"type":"function","title":"Parameters - Fresh.start_link/4","doc":"* `uri` - The URI to connect to as a binary.\n* `module` - The module that implements the WebSocket client behaviour.\n* `state` - The initial state to be passed to the module when it starts.\n* `opts` - A list of options to configure the WebSocket connection. Refer to `t:option/0` for available options.","ref":"Fresh.html#start_link/4-parameters"},{"type":"function","title":"Example - Fresh.start_link/4","doc":"iex> Fresh.start_link(\"wss://example.com/socket\", Example.WebSocket, %{}, name: {:local, :ws_conn})\n    {:ok, #PID<0.233.0>}","ref":"Fresh.html#start_link/4-example"},{"type":"type","title":"Fresh.connection_handle_response/0","doc":"Represents the response for all connection handle callbacks.","ref":"Fresh.html#t:connection_handle_response/0"},{"type":"type","title":"Fresh.control_frame/0","doc":"Represents control frames in a WebSocket connection.","ref":"Fresh.html#t:control_frame/0"},{"type":"type","title":"Fresh.data_frame/0","doc":"Represents data frames in a WebSocket connection.","ref":"Fresh.html#t:data_frame/0"},{"type":"type","title":"Fresh.error/0","doc":"Represents various error scenarios that can occur during WebSocket communication.","ref":"Fresh.html#t:error/0"},{"type":"type","title":"Fresh.generic_handle_response/0","doc":"Represents the response of a generic callback and enables you to manage the state.","ref":"Fresh.html#t:generic_handle_response/0"},{"type":"type","title":"Fresh.option/0","doc":"Represents optional configurations for WebSocket connections. Available options include:\n\n* `:name` - Registers a name for the WebSocket connection, allowing you to refer to it later using a name.\n* `:headers` - Specifies a list of headers to include in the WebSocket connection request. These headers will be sent during the connection upgrade.\n* `:transport_opts` - Additional options to pass to the transport layer used for the WebSocket connection. Consult the [Mint.HTTP documentation](https://hexdocs.pm/mint/Mint.HTTP.html#connect/4-options) for more informations.\n* `:mint_upgrade_opts` - Extra options to provide to [Mint.WebSocket](https://github.com/elixir-mint/mint_web_socket) during the WebSocket upgrade process. Consult the [Mint.WebSocket documentation](https://hexdocs.pm/mint_web_socket/Mint.WebSocket.html#upgrade/5-options) for additional information.\n* `:ping_interval` - This option is used for keeping the WebSocket connection alive by sending empty ping frames at regular intervals, specified in milliseconds. The default value is `30000` (30 seconds). To disable ping frames, set this option to `0`.\n* `:error_logging` - Allows toggling logging for error messages. Enabled by default.\n* `:info_logging` - Allows toggling logging for informational messages. Enabled by default.\n* `:backoff_initial` - Specifies the initial backoff time, in milliseconds, used between reconnection attempts. The default value is `250` (250 milliseconds).\n* `:backoff_max` - Sets the maximum time interval, in milliseconds, used between reconnection attempts. The default value is `30000` (30 seconds).\n* `:hibernate_after` - Specifies a timeout value, in milliseconds, which the WebSocket connection process will enter hibernation if there is no activity. Hibernation is disabled by default.","ref":"Fresh.html#t:option/0"},{"type":"type","title":"Example Configuration - Fresh.option/0","doc":"[\n      name: {:local, Example.Connection},\n      headers: [{\"Authorization\", \"Bearer token\"}],\n      ping_interval: 60_000,\n      error_logging: false,\n      backoff_initial: 5_000,\n      backoff_max: 60_000,\n      hibernate_after: 600_000\n    ]","ref":"Fresh.html#t:option/0-example-configuration"},{"type":"type","title":"Fresh.state/0","doc":"Represents the state of the given module, which can be anything.","ref":"Fresh.html#t:state/0"},{"type":"extras","title":"Changelog","doc":"# Changelog\n\nThis changelog documents all noteworthy changes in the project.","ref":"changelog.html"},{"type":"extras","title":"Next - Changelog","doc":"","ref":"changelog.html#next"},{"type":"extras","title":"v0.4.4 - 13th April 2023 - Changelog","doc":"- Just a small bump, nothing actually.","ref":"changelog.html#v0-4-4-13th-april-2023"},{"type":"extras","title":"v0.4.3 - 26th October 2023 - Changelog","doc":"","ref":"changelog.html#v0-4-3-26th-october-2023"},{"type":"extras","title":"Fixed - Changelog","doc":"- Failing to connect when no path is given in URI.","ref":"changelog.html#fixed"},{"type":"extras","title":"v0.4.2 - 12th October 2023 - Changelog","doc":"","ref":"changelog.html#v0-4-2-12th-october-2023"},{"type":"extras","title":"Added - Changelog","doc":"- New `open?/1` function, which allows you to check whether a connection is open.","ref":"changelog.html#added"},{"type":"extras","title":"v0.4.1 - 29th September 2023 - Changelog","doc":"","ref":"changelog.html#v0-4-1-29th-september-2023"},{"type":"extras","title":"Changed - Changelog","doc":"- The payload for `:reply` has been refined to `Mint.WebSocket.frame() | [Mint.WebSocket.frame()]`, meaning it can now be either a single WebSocket frame or a list of WebSocket frames.","ref":"changelog.html#changed"},{"type":"extras","title":"v0.4.0 - 23rd September 2023 - Changelog","doc":"","ref":"changelog.html#v0-4-0-23rd-september-2023"},{"type":"extras","title":"Added - Changelog","doc":"- Exponential backoff strategy for reconnection attempts.\n- `:hibernate_after` Option to automatically hibernate the WebSocket connection process after a specified duration of inactivity, helping conserve system resources.","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- Enhanced documentation for a more user-friendly experience.","ref":"changelog.html#changed"},{"type":"extras","title":"v0.3.0 - 22nd September 2023 - Changelog","doc":"","ref":"changelog.html#v0-3-0-22nd-september-2023"},{"type":"extras","title":"Added - Changelog","doc":"- Introduced `{:close, code, reason, state}` response for the generic callback, simplifying the process of sending a close frame to the server.\n- Added the `:info_logging` option, allowing you to toggle information messages.","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- Now, disconnection messages are logged as information.","ref":"changelog.html#changed"},{"type":"extras","title":"v0.2.1 - 17th September 2023 - Changelog","doc":"","ref":"changelog.html#v0-2-1-17th-september-2023"},{"type":"extras","title":"Added - Changelog","doc":"- Introduced the `close/3` function for easier sending of close frames.","ref":"changelog.html#added"},{"type":"extras","title":"v0.2.0 - 14th September 2023 - Changelog","doc":"","ref":"changelog.html#v0-2-0-14th-september-2023"},{"type":"extras","title":"Added - Changelog","doc":"- Implemented the `handle_terminate/2` callback for handling process termination.","ref":"changelog.html#added"},{"type":"extras","title":"Fixed - Changelog","doc":"- Addressed minor documentation issues.","ref":"changelog.html#fixed"},{"type":"extras","title":"v0.1.1 - 12th September 2023 - Changelog","doc":"","ref":"changelog.html#v0-1-1-12th-september-2023"},{"type":"extras","title":"Fixed - Changelog","doc":"- Added a queue for incoming messages while the websocket is `nil`.","ref":"changelog.html#fixed"},{"type":"extras","title":"v0.1.0 - 11th September 2023 - Changelog","doc":"","ref":"changelog.html#v0-1-0-11th-september-2023"},{"type":"extras","title":"Added - Changelog","doc":"- Included the missing `start/1` for the `__using__/1` macro.","ref":"changelog.html#added"},{"type":"extras","title":"v0.1.0-rc - 9th September 2023 - Changelog","doc":"- Initial release of Fresh.","ref":"changelog.html#v0-1-0-rc-9th-september-2023"},{"type":"extras","title":"Introduction","doc":"# Fresh\n\nFresh is an attempt to create a simple, reliable and flexible WebSocket client built atop the [Mint](https://github.com/elixir-mint) ecosystem 🌱","ref":"readme.html"},{"type":"extras","title":"Why Fresh? - Introduction","doc":"Discover the reasons behind choosing Fresh over existing libraries, summed up in three key aspects:\n\n- Simplicity\n- Resilience\n- Control","ref":"readme.html#why-fresh"},{"type":"extras","title":"Simplicity - Introduction","doc":"Fresh is designed with simplicity in mind, offering a user-friendly API that includes clear examples and comprehensive documentation.","ref":"readme.html#simplicity"},{"type":"extras","title":"Resilience - Introduction","doc":"Fresh drew significant inspiration from Redix, particularly in terms of resilience. By default, Fresh promptly re-establishing the connection when the server terminates the connection or encounters any connectivity issues. When used alongside Supervisor, Fresh delivers exceptional reliability.","ref":"readme.html#resilience"},{"type":"extras","title":"Control - Introduction","doc":"With Fresh, you gain extensive control over the flow of WebSocket connections, including control over resilience. You can manage scenarios requiring reconnection, identify those that should be ignored, specify when to gracefully terminate a connection, and even instruct Supervisor to restart your process as needed. Additionally, Fresh allows you to leverage custom `Mint.WebSocket` options, transport layer options, custom headers, and more, providing you with the flexibility you require.","ref":"readme.html#control"},{"type":"extras","title":"Installation - Introduction","doc":"Package can be installed by adding `fresh` to your list of dependencies in `mix.exs`:\n\n```elixir\ndefp deps do\n  [\n    {:fresh, \"~> 0.4.4\"}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Compatibility - Introduction","doc":"This library is well-tested with the following versions of Elixir and Erlang/OTP:\n\n- Elixir 1.14 or newer\n- Erlang/OTP 25 or newer\n\nWhile it may also work with older versions, we strongly recommend using the specified minimum versions for the best experience.","ref":"readme.html#compatibility"},{"type":"extras","title":"Example - Introduction","doc":"Below is an example of a WebSocket client that handles incoming frames to implement a simple counter:\n\n_It is worth mentioning that this example module contains approximately 15 lines of code._\n\n```elixir\ndefmodule EchoWebSocket do\n  use Fresh\n\n  def handle_connect(_status, _headers, _state) do\n    IO.puts(\"Start counting from 0\")\n    {:reply, {:text, \"1\"}, 0}\n  end\n\n  def handle_in({:text, number}, _state) do\n    number = String.to_integer(number)\n\n    IO.puts(\"Number: #{number}\")\n    {:reply, {:text, \"#{number + 1}\"}, number}\n  end\nend\n```\n\nYou can find more examples inside the [examples/](https://github.com/bunopnu/fresh/tree/main/examples) folder.","ref":"readme.html#example"},{"type":"extras","title":"Documentation - Introduction","doc":"For in-depth information and a detailed API reference, please consult the [HexDocs](https://hexdocs.pm/fresh).","ref":"readme.html#documentation"},{"type":"extras","title":"License - Introduction","doc":"Fresh is licensed under the MIT license.","ref":"readme.html#license"},{"type":"extras","title":"License","doc":"Copyright (c) 2024 bunopnu   (https://bun.rip)\n\nPermission is hereby granted, free of charge, to any person obtaining a copy\nof this software and associated documentation files (the \"Software\"), to deal\nin the Software without restriction, including without limitation the rights\nto use, copy, modify, merge, publish, distribute, sublicense, and/or sell\ncopies of the Software, and to permit persons to whom the Software is\nfurnished to do so, subject to the following conditions:\n\nThe above copyright notice and this permission notice shall be included in all\ncopies or substantial portions of the Software.\n\nTHE SOFTWARE IS PROVIDED \"AS IS\", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\nIMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\nFITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\nAUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\nLIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,\nOUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE\nSOFTWARE.","ref":"license.html"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,50,46,49]}}